/*******************************************************************
 *
 *	File:		CArlo.h
 *
 *	Author:		Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __CARLO_H
#define __CARLO_H

/*
 * Pagetable size is: 3 * PAGES_SIZE * SIZEOF(UINT) 
 */
#define BOOT_STRING_SIZE			64
#define PAGE_SIZE					4096
#define NR_PAGE_TABLE_ENTRIES		((PAGE_SIZE << 1) - 2)

class RArLo;
class CArloBase;

class ArloFactory
{
public:
	static CArloBase* CreateArloLC(RArLo& aArlo, const TDesC& aMachineName);
};


class CArloBase : public CBase
{
public:
	virtual void ConstructL();
	~CArloBase();

	TUint32	ArloLinear2Physical(TUint32 aLinearAdr) {return PlatLinear2Physical(aLinearAdr);}
	TUint	ArloExamine(TUint32 aAddressToExamine) {return PlatExamine(aAddressToExamine);}
	void	ArloBoot();
	TUint	ArloGetProcId() {return GetProcId();}
	void	SetBootString(TAny* aBootString, TInt aLength);
	void	AddPagesL(TInt aBlocks, TUint32 aLoadAddr);
	const TDesC& MachineName() {return iMachineName;}
	TBool	HasLoadedFile(){return iArloPageTableIndex > 2;}
protected:
	CArloBase(RArLo& aArlo);		// only ArloFactory constructs us
	void SetupArloPageTableL();

	virtual TUint32	PlatLinear2Physical(TUint32 aLinearAdr) = 0;
	virtual TUint	PlatExamine(TUint32 aAddressToExamine) = 0;
	virtual void	PlatBoot(TUint32 aPhysMapAdr, TUint32 aBootAdr) = 0;
	virtual TUint	GetProcId();

protected:
	TBool				iHasSetupArloPageTable;
	TInt				iArloPageTableIndex;
	TText8*				iBootString;		// we own this
	TUint32*			iArloPageTable;
	RArLo&				iArlo;
	TInfoName			iMachineName;
	friend class ArloFactory;
private:
	TText8*				iTable;				// we own this
};


class CArloSeries5 : public CArloBase
{
protected:
	CArloSeries5(RArLo& aArlo) : CArloBase(aArlo){} 
	virtual TUint	PlatExamine(TUint32 aAddressToExamine);
	virtual void	PlatBoot(TUint32 aPhysMapAdr, TUint32 aBootAdr);
	virtual TUint32	PlatLinear2Physical(TUint32 aLinearAdr);
	friend class ArloFactory;
};


class CArloArm4 : public CArloBase
{
protected:
	CArloArm4(RArLo& aArlo) : CArloBase(aArlo){} 
	virtual TUint32	PlatLinear2Physical(TUint32 aLinearAdr);
	virtual TUint	PlatExamine(TUint32 aAddressToExamine);
};

class CArloSeries5Mx : public CArloArm4
{
protected:
	CArloSeries5Mx(RArLo& aArlo) : CArloArm4(aArlo){} 
	virtual void	PlatBoot(TUint32 aPhysMapAdr, TUint32 aBootAdr);
	friend class ArloFactory;
};

class CArloSeries7 : public CArloArm4
{
protected:
	CArloSeries7(RArLo& aArlo) : CArloArm4(aArlo){} 
	virtual void	PlatBoot(TUint32 aPhysMapAdr, TUint32 aBootAdr);
	friend class ArloFactory;
};


class CArloWins : public CArloBase
{
protected:
	CArloWins(RArLo& aArlo) : CArloBase(aArlo){} 
	virtual TUint32	PlatLinear2Physical(TUint32 aLinearAdr){return aLinearAdr;}
	virtual TUint	PlatExamine(TUint32 aAddressToExamine){ return aAddressToExamine;}
	virtual void	PlatBoot(TUint32 aPhysMapAdr, TUint32 aBootAdr);
	friend class ArloFactory;
};





#endif			/* __CARLO_H */
